import os
import sys
import unittest

import six
from mock import patch
import pytest

from conans import DEFAULT_REVISION_V1
from conans.model.manifest import FileTreeManifest
from conans.model.package_metadata import PackageMetadata
from conans.model.ref import ConanFileReference, PackageReference
from conans.paths import BUILD_FOLDER, CONANINFO, CONAN_MANIFEST, EXPORT_FOLDER, \
    PACKAGES_FOLDER, SRC_FOLDER
from conans.server.store.server_store import ServerStore
from conans.test.utils.test_files import temp_folder
from conans.test.utils.tools import NO_SETTINGS_PACKAGE_ID, TestClient, TestServer, GenConanfile
from conans.util.env_reader import get_env
from conans.util.files import load


class RemoveLocksTest(unittest.TestCase):
    def test_remove_locks(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile().with_name("Hello").with_version("0.1")})
        client.run("create . lasote/testing")
        self.assertNotIn('does not contain a number!', client.out)
        ref = ConanFileReference.loads("Hello/0.1@lasote/testing")
        conan_folder = client.cache.package_layout(ref).base_folder()
        self.assertIn("locks", os.listdir(conan_folder))
        self.assertTrue(os.path.exists(conan_folder + ".count"))
        self.assertTrue(os.path.exists(conan_folder + ".count.lock"))
        client.run("remove * --locks", assert_error=True)
        self.assertIn("ERROR: Specifying a pattern is not supported", client.out)
        client.run("remove", assert_error=True)
        self.assertIn('ERROR: Please specify a pattern to be removed ("*" for all)', client.out)
        client.run("remove --locks")
        self.assertNotIn("locks", os.listdir(conan_folder))
        self.assertFalse(os.path.exists(conan_folder + ".count"))
        self.assertFalse(os.path.exists(conan_folder + ".count.lock"))


class RemoveRegistryTest(unittest.TestCase):

    def test_remove_registry(self):
        test_server = TestServer(users={"lasote": "password"})  # exported users and passwords
        servers = {"default": test_server}
        client = TestClient(servers=servers, users={"default": [("lasote", "password")]})
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . Test/0.1@lasote/testing")
        client.run("upload * --all --confirm")
        client.run('remove "*" -f')
        client.run("remote list_pref Test/0.1@lasote/testing")
        self.assertNotIn("Test/0.1@lasote/testing", client.out)
        registry_content = load(client.cache.remotes_path)
        self.assertNotIn("Test/0.1@lasote/testing", registry_content)


class RemoveOutdatedTest(unittest.TestCase):

    def test_remove_query(self):
        test_server = TestServer(users={"lasote": "password"})  # exported users and passwords
        servers = {"default": test_server}
        client = TestClient(servers=servers, users={"default": [("lasote", "password")]})
        conanfile = """from conans import ConanFile
class Test(ConanFile):
    settings = "os"
    """
        client.save({"conanfile.py": conanfile})
        client.run("create . Test/0.1@lasote/testing -s os=Windows")
        client.run("create . Test/0.1@lasote/testing -s os=Linux")
        client.save({"conanfile.py": conanfile.replace("settings", "pass #")})
        client.run("create . Test2/0.1@lasote/testing")
        client.run("upload * --all --confirm")
        for remote in ("", "-r=default"):
            client.run("remove Test/0.1@lasote/testing -q=os=Windows -f %s" % remote)
            client.run("search Test/0.1@lasote/testing %s" % remote)
            self.assertNotIn("os: Windows", client.out)
            self.assertIn("os: Linux", client.out)

            client.run("remove Test2/0.1@lasote/testing -q=os=Windows -f %s" % remote)
            client.run("search Test2/0.1@lasote/testing %s" % remote)
            self.assertIn("Package_ID: %s" % NO_SETTINGS_PACKAGE_ID, client.out)
            client.run("remove Test2/0.1@lasote/testing -q=os=None -f %s" % remote)
            client.run("search Test2/0.1@lasote/testing %s" % remote)
            self.assertNotIn("Package_ID: %s" % NO_SETTINGS_PACKAGE_ID, client.out)
            self.assertIn("There are no packages", client.out)

    @pytest.mark.skipif(get_env("TESTING_REVISIONS_ENABLED", False), reason="No sense with revs")
    def test_remove_outdated(self):
        test_server = TestServer(users={"lasote": "password"})  # exported users and passwords
        servers = {"default": test_server}
        client = TestClient(servers=servers, users={"default": [("lasote", "password")]})
        conanfile = """from conans import ConanFile
class Test(ConanFile):
    name = "Test"
    version = "0.1"
    settings = "os"
    """
        client.save({"conanfile.py": conanfile})
        client.run("export . lasote/testing")
        client.run("install Test/0.1@lasote/testing --build -s os=Windows")
        client.save({"conanfile.py": "# comment\n%s" % conanfile})
        client.run("export . lasote/testing")
        client.run("install Test/0.1@lasote/testing --build -s os=Linux")
        client.run("upload * --all --confirm")
        for remote in ("", "-r=default"):
            client.run("search Test/0.1@lasote/testing %s" % remote)
            self.assertIn("os: Windows", client.out)
            self.assertIn("os: Linux", client.out)
            client.run("remove Test/0.1@lasote/testing -p --outdated -f %s" % remote)
            client.run("search Test/0.1@lasote/testing  %s" % remote)
            self.assertNotIn("os: Windows", client.out)
            self.assertIn("os: Linux", client.out)


fake_recipe_hash = "999999999"
conaninfo = '''
[settings]
    arch=x64
    os=Windows
    compiler=Visual Studio
    compiler.version=8.%s
[options]
    use_Qt=True
[full_requires]
  Hello2/0.1@lasote/stable:11111
  OpenSSL/2.10@lasote/testing:2222
  HelloInfo1/0.45@myuser/testing:33333
[recipe_hash]
''' + fake_recipe_hash + '''
[recipe_revision]
'''


class RemoveTest(unittest.TestCase):

    def setUp(self):
        test_conanfile_contents = str(GenConanfile("Hello", "0.1"))
        hello_files = {"conanfile.py": test_conanfile_contents}

        self.server_folder = temp_folder()
        test_server = TestServer(users={"myuser": "mypass"},
                                 base_path=self.server_folder)  # exported users and passwords
        self.server = test_server
        servers = {"default": test_server}
        client = TestClient(servers=servers, users={"default": [("myuser", "mypass")]})

        # Conans with and without packages created
        self.root_folder = {"H1": 'Hello/1.4.10@myuser/testing',
                            "H2": 'Hello/2.4.11@myuser/testing',
                            "B": 'Bye/0.14@myuser/testing',
                            "O": 'Other/1.2@myuser/testing'}

        files = {}
        prefs = []
        for key, folder in self.root_folder.items():
            ref = ConanFileReference.loads(folder)
            folder = folder.replace("@", "/")
            fake_metadata = PackageMetadata()
            fake_metadata.recipe.revision = DEFAULT_REVISION_V1
            files["%s/%s/conanfile.py" % (folder, EXPORT_FOLDER)] = test_conanfile_contents
            files["%s/%s/conanmanifest.txt" % (
                folder, EXPORT_FOLDER)] = "%s\nconanfile.py: 234234234" % fake_recipe_hash
            files["%s/%s/conans.txt" % (folder, SRC_FOLDER)] = ""
            for pack_id in (1, 2):
                i = pack_id
                pack_id = "%s_%s" % (pack_id, key)
                fake_metadata.packages[pack_id].revision = DEFAULT_REVISION_V1
                prefs.append(PackageReference(ref, str(pack_id)))
                files["%s/%s/%s/conans.txt" % (folder, BUILD_FOLDER, pack_id)] = ""
                files["%s/%s/%s/conans.txt" % (folder, PACKAGES_FOLDER, pack_id)] = ""
                files[
                    "%s/%s/%s/%s" % (folder, PACKAGES_FOLDER, pack_id, CONANINFO)] = conaninfo % str(
                    i) + "905eefe3570dd09a8453b30b9272bb44"
                files["%s/%s/%s/%s" % (folder, PACKAGES_FOLDER, pack_id, CONAN_MANIFEST)] = ""
            files["%s/metadata.json" % folder] = fake_metadata.dumps()
            exports_sources_dir = client.cache.package_layout(ref).export_sources()
            os.makedirs(exports_sources_dir)

        client.save(files, client.cache.store)

        # Create the manifests to be able to upload
        for pref in prefs:
            pkg_folder = client.cache.package_layout(pref.ref).package(pref)
            expected_manifest = FileTreeManifest.create(pkg_folder)
            files["%s/%s/%s/%s" % (pref.ref.dir_repr(),
                                   PACKAGES_FOLDER,
                                   pref.id,
                                   CONAN_MANIFEST)] = repr(expected_manifest)

        client.save(files, client.cache.store)

        self.client = client

        for folder in self.root_folder.values():
            client.run("upload %s --all" % folder)

        self.assert_folders({"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            {"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            {"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            {"H1": True, "H2": True, "B": True, "O": True})

    def assert_folders(self, local_folders, remote_folders, build_folders, src_folders):
        for base_path, folders in [(self.client.cache, local_folders),
                                   (self.server.server_store, remote_folders)]:
            root_folder = base_path.store
            for k, shas in folders.items():
                folder = os.path.join(root_folder, self.root_folder[k].replace("@", "/"))
                ref = ConanFileReference.loads(self.root_folder[k])
                if isinstance(base_path, ServerStore):
                    if self.client.cache.config.revisions_enabled:
                        try:
                            rev = self.client.cache.package_layout(ref).recipe_revision()
                        except:
                            # This whole test is a crap, we cannot guess remote revision
                            # if the package is not in local anymore
                            continue
                    else:
                        rev = DEFAULT_REVISION_V1
                    folder += "/%s" % rev
                if shas is None:
                    self.assertFalse(os.path.exists(folder))
                else:
                    for value in (1, 2):
                        sha = "%s_%s" % (value, k)
                        package_folder = os.path.join(folder, "package", sha)
                        if isinstance(base_path, ServerStore):
                            if self.client.cache.config.revisions_enabled:
                                pref = PackageReference(ref, sha)
                                try:
                                    layout = self.client.cache.package_layout(pref.ref)
                                    prev = layout.package_revision(pref)
                                except:
                                    # This whole test is a crap, we cannot guess remote revision
                                    # if the package is not in local anymore
                                    continue
                            else:
                                prev = DEFAULT_REVISION_V1
                            package_folder += "/%s" % prev if prev else ""
                        if value in shas:
                            self.assertTrue(os.path.exists(package_folder),
                                            "%s doesn't exist " % package_folder)
                        else:
                            self.assertFalse(os.path.exists(package_folder), package_folder)

        root_folder = self.client.cache.store
        for k, shas in build_folders.items():
            folder = os.path.join(root_folder, self.root_folder[k].replace("@", "/"))
            if shas is None:
                self.assertFalse(os.path.exists(folder))
            else:
                for value in (1, 2):
                    sha = "%s_%s" % (value, k)
                    build_folder = os.path.join(folder, "build", sha)
                    if value in shas:
                        self.assertTrue(os.path.exists(build_folder))
                    else:
                        self.assertFalse(os.path.exists(build_folder))
        for k, value in src_folders.items():
            folder = os.path.join(root_folder, self.root_folder[k].replace("@", "/"), "source")
            if value:
                self.assertTrue(os.path.exists(folder))
            else:
                self.assertFalse(os.path.exists(folder))

    def test_basic(self):
        self.client.run("remove hello/* -f")
        self.assert_folders(local_folders={"H1": None, "H2": None, "B": [1, 2], "O": [1, 2]},
                            remote_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            build_folders={"H1": None, "H2": None, "B": [1, 2], "O": [1, 2]},
                            src_folders={"H1": False, "H2": False, "B": True, "O": True})
        folders = os.listdir(self.client.storage_folder)
        six.assertCountEqual(self, ["Other", "Bye"], folders)

    def test_basic_mocked(self):
        with patch.object(sys.stdin, "readline", return_value="y"):
            self.client.run("remove hello/*")
        self.assert_folders(local_folders={"H1": None, "H2": None, "B": [1, 2], "O": [1, 2]},
                            remote_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            build_folders={"H1": None, "H2": None, "B": [1, 2], "O": [1, 2]},
                            src_folders={"H1": False, "H2": False, "B": True, "O": True})
        folders = os.listdir(self.client.storage_folder)
        six.assertCountEqual(self, ["Other", "Bye"], folders)

    def test_basic_packages(self):
        self.client.run("remove hello/* -p -f")
        self.assert_folders(local_folders={"H1": [], "H2": [], "B": [1, 2], "O": [1, 2]},
                            remote_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            build_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            src_folders={"H1": True, "H2": True, "B": True, "O": True})
        folders = os.listdir(self.client.storage_folder)
        six.assertCountEqual(self, ["Hello", "Other", "Bye"], folders)
        six.assertCountEqual(self, ["build", "source", "export", "export_source", "metadata.json",
                                    "dl", "metadata.json.lock"],
                             os.listdir(os.path.join(self.client.storage_folder,
                                                     "Hello/1.4.10/myuser/testing")))
        six.assertCountEqual(self, ["build", "source", "export", "export_source", "metadata.json",
                                    "dl", "metadata.json.lock"],
                             os.listdir(os.path.join(self.client.storage_folder,
                                                     "Hello/2.4.11/myuser/testing")))

    def _validate_remove_all_hello_packages(self):
        self.assert_folders(local_folders={"H1": None, "H2": None, "B": [1, 2], "O": [1, 2]},
                            remote_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            build_folders={"H1": None, "H2": None, "B": [1, 2], "O": [1, 2]},
                            src_folders={"H1": False, "H2": False, "B": True, "O": True})
        folders = os.listdir(self.client.storage_folder)
        six.assertCountEqual(self, ["Other", "Bye"], folders)

    def test_remove_any_package_version(self):
        self.client.run("remove Hello/*@myuser/testing -f")
        self._validate_remove_all_hello_packages()

    def test_remove_any_package_version_user(self):
        self.client.run("remove Hello/*@*/testing -f")
        self._validate_remove_all_hello_packages()

    def test_remove_any_package_version_channel(self):
        self.client.run("remove Hello/*@*/* -f")
        self._validate_remove_all_hello_packages()

    def _validate_remove_hello_1_4_10(self):
        self.assert_folders(local_folders={"H1": None, "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            remote_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            build_folders={"H1": None, "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            src_folders={"H1": False, "H2": True, "B": True, "O": True})
        folders = os.listdir(self.client.storage_folder)
        six.assertCountEqual(self, ["Hello", "Other", "Bye"], folders)

    def test_remove_any_package_user(self):
        self.client.run("remove Hello/1.4.10@*/testing -f")
        self._validate_remove_hello_1_4_10()

    def test_remove_any_package_channel(self):
        self.client.run("remove Hello/1.4.10@myuser/* -f")
        self._validate_remove_hello_1_4_10()

    def test_builds(self):
        with patch.object(sys.stdin, "readline", return_value="y"):
            self.client.run("remove hello/* -b")
        self.assert_folders(local_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            remote_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            build_folders={"H1": [], "H2": [], "B": [1, 2], "O": [1, 2]},
                            src_folders={"H1": True, "H2": True, "B": True, "O": True})
        folders = os.listdir(self.client.storage_folder)
        six.assertCountEqual(self, ["Hello", "Other", "Bye"], folders)
        six.assertCountEqual(self, ["package", "dl", "source", "export", "export_source",
                                    "metadata.json", "metadata.json.lock"],
                             os.listdir(os.path.join(self.client.storage_folder,
                                                     "Hello/1.4.10/myuser/testing")))
        six.assertCountEqual(self, ["package", "dl", "source", "export", "export_source",
                                    "metadata.json", "metadata.json.lock"],
                             os.listdir(os.path.join(self.client.storage_folder,
                                                     "Hello/2.4.11/myuser/testing")))

    def test_src(self):
        with patch.object(sys.stdin, "readline", return_value="y"):
            self.client.run("remove hello/* -s")
        self.assert_folders(local_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            remote_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            build_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            src_folders={"H1": False, "H2": False, "B": True, "O": True})
        folders = os.listdir(self.client.storage_folder)
        six.assertCountEqual(self, ["Hello", "Other", "Bye"], folders)
        six.assertCountEqual(self, ["package", "build", "export", "export_source", "metadata.json",
                                    "dl", "metadata.json.lock"],
                             os.listdir(os.path.join(self.client.storage_folder,
                                                     "Hello/1.4.10/myuser/testing")))
        six.assertCountEqual(self, ["package", "build", "export", "export_source", "metadata.json",
                                    "dl", "metadata.json.lock"],
                             os.listdir(os.path.join(self.client.storage_folder,
                                                     "Hello/2.4.11/myuser/testing")))

    def test_reject_removal(self):
        with patch.object(sys.stdin, "readline", return_value="n"):
            self.client.run("remove hello/* -s -b -p")
        self.assert_folders(local_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            remote_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            build_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            src_folders={"H1": True, "H2": True, "B": True, "O": True})

    def test_remote_build_error(self):
        self.client.run("remove hello/* -b -r=default", assert_error=True)
        self.assertIn("Remotes don't have 'build' or 'src' folder", self.client.out)
        self.assert_folders(local_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            remote_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            build_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            src_folders={"H1": True, "H2": True, "B": True, "O": True})

    def test_remote_packages(self):
        self.client.run("remove hello/* -p -r=default -f")
        self.assert_folders(local_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            remote_folders={"H1": [], "H2": [], "B": [1, 2], "O": [1, 2]},
                            build_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            src_folders={"H1": True, "H2": True, "B": True, "O": True})

    def test_remote_conans(self):
        self.client.run("remove hello/* -r=default -f")
        self.assert_folders(local_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            remote_folders={"H1": None, "H2": None, "B": [1, 2], "O": [1, 2]},
                            build_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            src_folders={"H1": True, "H2": True, "B": True, "O": True})
        remote_folder = os.path.join(self.server_folder, ".conan_server/data")
        folders = os.listdir(remote_folder)
        six.assertCountEqual(self, ["Other", "Bye"], folders)

    def test_remove_specific_package(self):
        self.client.run("remove hello/1.4.10* -p=1_H1 -f")
        self.assert_folders(local_folders={"H1": [2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            remote_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            build_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            src_folders={"H1": True, "H2": True, "B": True, "O": True})

    def test_remove_specific_packages(self):
        self.client.run("remove hello/1.4.10* -p=1_H1 -p 2_H1 -f")
        self.assert_folders(local_folders={"H1": [], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            remote_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            build_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            src_folders={"H1": True, "H2": True, "B": True, "O": True})

    def test_remove_specific_build(self):
        self.client.run("remove hello/1.4.10* -b=1_H1 -f")
        self.assert_folders(local_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            remote_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            build_folders={"H1": [2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            src_folders={"H1": True, "H2": True, "B": True, "O": True})

    def test_remove_specific_builds(self):
        self.client.run("remove hello/1.4.10* -b=1_H1 -b=2_H1 -f")
        self.assert_folders(local_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            remote_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            build_folders={"H1": [], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            src_folders={"H1": True, "H2": True, "B": True, "O": True})

    def test_remove_remote_specific_package(self):
        self.client.run("remove hello/1.4.10* -p=1_H1 -f -r=default")
        self.assert_folders(local_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            remote_folders={"H1": [2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            build_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            src_folders={"H1": True, "H2": True, "B": True, "O": True})

    def test_remove_remote_specific_packages(self):
        self.client.run("remove hello/1.4.10* -p=1_H1 -p2_H1 -f -r=default")
        self.assert_folders(local_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            remote_folders={"H1": [], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            build_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            src_folders={"H1": True, "H2": True, "B": True, "O": True})

    def test_try_remove_using_query_and_packages_or_builds(self):
        with six.assertRaisesRegex(self, Exception, "Command failed"):
            self.client.run("remove hello/1.4.10@lasote/stable -p=1_H1 -q 'compiler.version=4.8' ")
            self.assertIn("'-q' and '-p' parameters can't be used at the same time", self.client.out)

        with six.assertRaisesRegex(self, Exception, "Command failed"):
            self.client.run("remove hello/1.4.10@lasote/stable -b=1_H1 -q 'compiler.version=4.8' ")
            self.assertIn("'-q' and '-b' parameters can't be used at the same time", self.client.out)

    @pytest.mark.skipif(get_env("TESTING_REVISIONS_ENABLED", False), reason="This test is insane to be "
                                                                            "tested with revisions, in "
                                                                            "general all the module")
    def test_query_remove_locally(self):
        self.client.run("remove notfoundname/1.4.10@myuser/testing -q='compiler.version=4.4' -f",
                        assert_error=True)
        self.assertIn("Recipe not found: 'notfoundname/1.4.10@myuser/testing'", self.client.out)
        self.assert_folders({"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            {"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            {"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            {"H1": True, "H2": True, "B": True, "O": True})

        self.client.run('remove Hello/1.4.10@myuser/testing -q="compiler.version=8.1" -f')
        self.assertNotIn("No packages matching the query", self.client.out)
        self.assert_folders(local_folders={"H1": [2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            remote_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            build_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            src_folders={"H1": True, "H2": True, "B": True, "O": True})

        self.client.run('remove Hello/1.4.10@myuser/testing -q="compiler.version=8.2" -f')
        self.assertNotIn("No packages matching the query", self.client.out)
        self.assert_folders(local_folders={"H1": [], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            remote_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            build_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            src_folders={"H1": True, "H2": True, "B": True, "O": True})

        self.client.run('remove Hello/1.4.10@myuser/testing -q="compiler.version=8.2" -f -r default')
        self.assertNotIn("No packages matching the query", self.client.out)
        self.assert_folders(local_folders={"H1": [], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            remote_folders={"H1": [1], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            build_folders={"H1": [1, 2], "H2": [1, 2], "B": [1, 2], "O": [1, 2]},
                            src_folders={"H1": True, "H2": True, "B": True, "O": True})


class RemoveWithoutUserChannel(unittest.TestCase):

    def setUp(self):
        self.test_server = TestServer(users={"lasote": "password"},
                                      write_permissions=[("lib/1.0@*/*", "lasote")])
        servers = {"default": self.test_server}
        self.client = TestClient(servers=servers, users={"default": [("lasote", "password")]})

    def test_local(self):
        self.client.save({"conanfile.py": GenConanfile()})
        self.client.run("create . lib/1.0@")
        self.client.run("remove lib/1.0 -f")
        folder = self.client.cache.package_layout(ConanFileReference.loads("lib/1.0@")).export()
        self.assertFalse(os.path.exists(folder))

    def test_remote(self):
        self.client.save({"conanfile.py": GenConanfile()})
        self.client.run("create . lib/1.0@")
        self.client.run("upload lib/1.0 -r default -c --all")
        self.client.run("remove lib/1.0 -f")
        # we can still install it
        self.client.run("install lib/1.0@")
        self.assertIn("Installing package: lib/1.0", self.client.out)
        self.client.run("remove lib/1.0 -f")

        # Now remove remotely
        self.client.run("remove lib/1.0 -f -r default")
        self.client.run("install lib/1.0@", assert_error=True)
        self.assertIn("ERROR: Unable to find 'lib/1.0' in remotes", self.client.out)


class RemovePackageRevisionsTest(unittest.TestCase):

    NO_SETTINGS_RREF = "f3367e0e7d170aa12abccb175fee5f97"

    def setUp(self):
        self.test_server = TestServer(users={"user": "password"},
                                      write_permissions=[("foobar/0.1@*/*", "user")])
        servers = {"default": self.test_server}
        self.client = TestClient(servers=servers, users={"default": [("user", "password")]})
        self.client.run("config set general.revisions_enabled=1")

    def test_remove_local_package_id_argument(self):
        """ Remove package ID based on recipe revision. The package must be deleted, but
            the recipe must be preserved
            Package ID is a separated argument: <package>#<rref> -p <pkgid>
        """
        self.client.save({"conanfile.py": GenConanfile()})
        self.client.run("create . foobar/0.1@user/testing")
        self.client.run("info foobar/0.1@user/testing")
        self.assertIn("Binary: Cache", self.client.out)
        self.assertIn("Revision: f3367e0e7d170aa12abccb175fee5f97", self.client.out)
        self.assertIn("Package revision: 83c38d3b4e5f1b8450434436eec31b00", self.client.out)

        self.client.run("remove -f foobar/0.1@user/testing#{} -p {}"
                        .format(self.NO_SETTINGS_RREF, NO_SETTINGS_PACKAGE_ID))
        self.client.run("info foobar/0.1@user/testing")
        self.assertIn("Binary: Missing", self.client.out)
        self.assertIn("Revision: f3367e0e7d170aa12abccb175fee5f97", self.client.out)
        self.assertIn("Package revision: None", self.client.out)

    def test_remove_local_package_id_reference(self):
        """ Remove package ID based on recipe revision. The package must be deleted, but
            the recipe must be preserved.
            Package ID is part of package reference: <package>#<rref>:<pkgid>
        """
        self.client.save({"conanfile.py": GenConanfile()})
        self.client.run("create . foobar/0.1@user/testing")
        self.client.run("info foobar/0.1@user/testing")
        self.assertIn("Binary: Cache", self.client.out)
        self.assertIn("Revision: f3367e0e7d170aa12abccb175fee5f97", self.client.out)
        self.assertIn("Package revision: 83c38d3b4e5f1b8450434436eec31b00", self.client.out)

        self.client.run("remove -f foobar/0.1@user/testing#{}:{}"
                        .format(self.NO_SETTINGS_RREF, NO_SETTINGS_PACKAGE_ID))
        self.client.run("info foobar/0.1@user/testing")
        self.assertIn("Binary: Missing", self.client.out)
        self.assertIn("Revision: f3367e0e7d170aa12abccb175fee5f97", self.client.out)
        self.assertIn("Package revision: None", self.client.out)

    def test_remove_duplicated_package_id(self):
        """ The package ID must not be present in both -p argument and package reference
        """
        self.client.save({"conanfile.py": GenConanfile()})
        self.client.run("create . foobar/0.1@user/testing")
        self.client.run("remove -f foobar/0.1@user/testing#{}:{} -p {}"
                        .format(self.NO_SETTINGS_RREF, NO_SETTINGS_PACKAGE_ID,
                                NO_SETTINGS_PACKAGE_ID), assert_error=True)
        self.assertIn("Use package ID only as -p argument or reference, not both", self.client.out)

    def test_remove_remote_package_id_reference(self):
        """ Remove remote package ID based on recipe revision. The package must be deleted, but
            the recipe must be preserved.
            Package ID is part of package reference: <package>#<rref>:<pkgid>
        """
        self.client.save({"conanfile.py": GenConanfile()})
        self.client.run("create . foobar/0.1@user/testing")
        self.client.run("upload foobar/0.1@user/testing -r default -c --all")
        self.client.run("remove -f foobar/0.1@user/testing#{}:{}"
                        .format(self.NO_SETTINGS_RREF, NO_SETTINGS_PACKAGE_ID))
        self.client.run("info foobar/0.1@user/testing")
        self.assertIn("Binary: Download", self.client.out)
        self.client.run("remove -f foobar/0.1@user/testing#{}:{} -r default"
                        .format(self.NO_SETTINGS_RREF, NO_SETTINGS_PACKAGE_ID))
        self.client.run("info foobar/0.1@user/testing")
        self.assertIn("Binary: Missing", self.client.out)



import os

import pytest
import textwrap

from conans.test.assets.sources import gen_function_cpp
from conans.test.functional.toolchains.meson._base import TestMesonBase


@pytest.mark.tool_pkg_config
class MesonPkgConfigTest(TestMesonBase):
    _conanfile_py = textwrap.dedent("""
    from conan import ConanFile
    from conan.tools.meson import Meson, MesonToolchain


    class App(ConanFile):
        settings = "os", "arch", "compiler", "build_type"
        generators = "pkg_config"
        requires = "hello/0.1"

        def layout(self):
            self.folders.build = "build"

        def generate(self):
            tc = MesonToolchain(self)
            tc.generate()

        def build(self):
            meson = Meson(self)
            meson.configure()
            meson.build()
    """)

    _meson_build = textwrap.dedent("""
    project('tutorial', 'cpp')
    hello = dependency('hello', version : '>=0.1')
    executable('demo', 'main.cpp', dependencies: hello)
    """)

    def test_reuse(self):
        self.t.run("new hello/0.1 -s")
        self.t.run("create . hello/0.1@ %s" % self._settings_str)
        app = gen_function_cpp(name="main", includes=["hello"], calls=["hello"])
        # Prepare the actual consumer package
        self.t.save({"conanfile.py": self._conanfile_py,
                     "meson.build": self._meson_build,
                     "main.cpp": app},
                    clean_first=True)

        # Build in the cache
        self.t.run("install . %s" % self._settings_str)
        self.assertIn("conanfile.py: Generator pkg_config created hello.pc", self.t.out)
        self.t.run("build .")
        self.t.run_command(os.path.join("build", "demo"))

        self.assertIn("Hello World Release!", self.t.out)

        self._check_binary()



import platform
import textwrap
import unittest

import pytest

from conans.test.utils.tools import TestClient


class LibcxxSettingTest(unittest.TestCase):

    @pytest.mark.skipif(platform.system() == "Windows", reason="Not in Windows")
    @pytest.mark.tool_cmake
    def test_declared_stdlib_and_passed(self):
        file_content = textwrap.dedent('''
            from conans import ConanFile, CMake

            class ConanFileToolsTest(ConanFile):
                settings = "os", "compiler", "arch", "build_type"
                generators = ["cmake"]

                def build(self):
                    cmake = CMake(self)
                    self.output.warn(cmake.command_line)
                    self.run('cmake . %s' % cmake.command_line)
                    self.run("cmake --build . %s" %  cmake.build_config)
                ''')

        cmakelists = textwrap.dedent('''PROJECT(conanzlib)
            set(CONAN_DISABLE_CHECK_COMPILER TRUE)
            cmake_minimum_required(VERSION 2.8)
            include(conanbuildinfo.cmake)
            conan_basic_setup()
            message("CXX FLAGS=> ${CMAKE_CXX_FLAGS}")
            get_directory_property( DirDefs DIRECTORY ${CMAKE_SOURCE_DIR} COMPILE_DEFINITIONS)
            foreach( d ${DirDefs} )
                message( STATUS "Found Define: " ${d} )
            endforeach()
            ''')
        client = TestClient()
        client.save({"conanfile.py": file_content,
                     "CMakeLists.txt": cmakelists})
        client.run("export . pkg/0.1@lasote/testing")

        if platform.system() == "SunOS":
            client.run('install . -s compiler=sun-cc -s compiler.libcxx=libCstd')
            client.run('build .')
            self.assertIn("-library=Cstd", client.out)

            client.run('install -s compiler=sun-cc -s compiler.libcxx=libstdcxx')
            client.run('build .')
            self.assertIn("-library=stdcxx4", client.out)

            client.run('install . -s compiler=sun-cc -s compiler.libcxx=libstlport')
            client.run('build .')
            self.assertIn("-library=stlport4", client.out)
        else:
            client.run('install . -s compiler=clang -s compiler.version=3.3 '
                       '-s compiler.libcxx=libstdc++ ')
            client.run('build .')
            self.assertIn("-stdlib=libstdc++", client.out)
            self.assertIn("Found Define: _GLIBCXX_USE_CXX11_ABI=0", client.out)

            client.run('install . -s compiler=clang -s compiler.version=3.3 '
                       '-s compiler.libcxx=libstdc++11')
            client.run('build .')
            self.assertIn("-stdlib=libstdc++", client.out)
            self.assertIn("Found Define: _GLIBCXX_USE_CXX11_ABI=1", client.out)

            client.run('install . -s compiler=clang -s compiler.version=3.3 '
                       '-s compiler.libcxx=libc++')
            client.run('build .')
            self.assertIn("-stdlib=libc++", client.out)
            self.assertNotIn("Found Define: _GLIBCXX_USE_CXX11", client.out)

    def test_C_only(self):
        conanfile = """from conans import ConanFile

class ConanFileToolsTest(ConanFile):
    name = "test"
    version = "1.9"
    settings = "os", "compiler", "arch", "build_type"

    def configure(self):
        del self.settings.compiler.libcxx # C package only
    """

        client = TestClient()
        client.save({"conanfile.py": conanfile})
        # Check that it not fails the config method with Visual Studio, because of the lack of libcxx
        client.run('install . -s compiler="Visual Studio" -s compiler.version=14')
        self.assertIn("conanfile.py (test/1.9): Generated conaninfo.txt", client.out)

        conaninfo = client.load("conaninfo.txt")
        self.assertNotIn("libcxx", conaninfo)
        client.run('install . -s compiler=gcc -s compiler.version=4.9')
        conaninfo = client.load("conaninfo.txt")
        self.assertNotIn("libcxx", conaninfo)

        client.run("create . lasote/testing -s compiler=gcc -s compiler.version=4.9")

        # Now try to reuse the installed package defining libstc++11 for the new package
        newlib_content = '''
from conans import ConanFile, CMake

class ConanFileToolsTest(ConanFile):
    settings = "os", "compiler", "arch", "build_type"
    requires = "test/1.9@lasote/testing"
    '''

        client.save({"conanfile.py": newlib_content})
        client.run('install . -s compiler=gcc -s compiler.libcxx=libstdc++11 '
                   '-s compiler.version=4.9')
        # Package is found and everything is ok
        self.assertIn("test/1.9@lasote/testing: Already installed!", client.out)
        self.assertIn("conanfile.py: Generated conaninfo.txt", client.out)
        conaninfo = client.load("conaninfo.txt")
        self.assertIn("libcxx", conaninfo)
        client.run('install . -s compiler=gcc -s compiler.libcxx=libstdc++ -s compiler.version=4.9')
        # Package is found and everything is ok
        self.assertIn("test/1.9@lasote/testing: Already installed!", client.out)
        self.assertIn("conanfile.py: Generated conaninfo.txt", client.out)



# coding=utf-8

import argparse
import unittest

from parameterized.parameterized import parameterized_class

from conans.client.command import _add_profile_arguments


@parameterized_class([{"argument": ["env", "-e", "--env"]},
                      {"argument": ["options", "-o", "--options"]},
                      {"argument": ["profile", "-pr", "--profile"]},
                      {"argument": ["settings", "-s", "--settings"]},
                      {"argument": ["conf", "-c", "--conf"]}]
                     )
class ArgsParseProfileTest(unittest.TestCase):
    """ Check argparse for profile arguments """

    def setUp(self):
        self.item, self.short_arg, self.long_arg = self.argument
        self.args_dest_build = '{}_build'.format(self.item)
        self.args_dest_host = '{}_host'.format(self.item)

    def _run_parse(self, *args):
        parser = argparse.ArgumentParser()
        _add_profile_arguments(parser)
        parsed_args = parser.parse_args(*args)
        build = getattr(parsed_args, self.args_dest_build)
        host = getattr(parsed_args, self.args_dest_host)
        return build, host

    def test_default(self):
        """ The old '--settings', '--profile',... refers to the build machine """
        build, host = self._run_parse([self.long_arg, "it1"])
        self.assertListEqual(host, ["it1"])
        self.assertIsNone(build)

        build, host = self._run_parse([self.long_arg, "it1", self.short_arg, "it2"])
        self.assertListEqual(host, ["it1", "it2"])
        self.assertIsNone(build)

    def test_build_machine(self):
        """ If provided with build suffix (':b', ':build'), those correspond to the build machine """
        long_arg = "{}:build".format(self.long_arg)
        short_arg = "{}:b".format(self.short_arg)

        build, host = self._run_parse([long_arg, "it1"])
        self.assertListEqual(build, ["it1"])
        self.assertIsNone(host)

        build, host = self._run_parse([long_arg, "it1", short_arg, "it2"])
        self.assertListEqual(build, ["it1", "it2"])
        self.assertIsNone(host)

    def test_mix_old_and_host_machine(self):
        """ Old arguments and new ':host' ones are composable """
        new_long_arg = "{}:host".format(self.long_arg)
        new_short_arg = "{}:h".format(self.short_arg)

        build, host = self._run_parse([new_long_arg, "it1", self.long_arg, "it2",
                                       new_short_arg, "it3", self.short_arg, "it4"])
        self.assertListEqual(host, ["it1", "it2", "it3", "it4"])
        self.assertIsNone(build)

    def test_host_machine(self):
        """ If provided with host suffix (':h', ':host'), those correspond to the host machine """
        long_arg = "{}:host".format(self.long_arg)
        short_arg = "{}:h".format(self.short_arg)

        build, host = self._run_parse([long_arg, "it1"])
        self.assertListEqual(host, ["it1"])
        self.assertIsNone(build)

        build, host = self._run_parse([long_arg, "it1", short_arg, "it2"])
        self.assertListEqual(host, ["it1", "it2"])
        self.assertIsNone(build)

    def test_build_and_host(self):
        """ Of course, we can provide build and host in the same command line """
        build, host = self._run_parse(["{}:build".format(self.long_arg), "b1",
                                       "{}:host".format(self.long_arg), "h1"])
        self.assertListEqual(build, ["b1"])
        self.assertListEqual(host, ["h1"])



import os
import platform
import unittest

import pytest

from conans.model.ref import ConanFileReference
from conans.paths import PACKAGE_TGZ_NAME
from conans.test.utils.tools import TestServer, TurboTestClient


class CompressSymlinksZeroSize(unittest.TestCase):

    @pytest.mark.skipif(platform.system() != "Linux", reason="Only linux")
    def test_package_symlinks_zero_size(self):
        server = TestServer()
        client = TurboTestClient(servers={"default": server})

        conanfile = """
import os
from conans import ConanFile, tools

class HelloConan(ConanFile):

    def package(self):
        # Link to file.txt and then remove it
        tools.save(os.path.join(self.package_folder, "file.txt"), "contents")
        os.symlink("file.txt", os.path.join(self.package_folder, "link.txt"))
"""
        ref = ConanFileReference.loads("lib/1.0@conan/stable")
        # By default it is not allowed
        pref = client.create(ref, conanfile=conanfile)
        # Upload, it will create the tgz
        client.upload_all(ref)

        # We can uncompress it without warns
        p_folder = client.cache.package_layout(pref.ref).download_package(pref)
        tgz = os.path.join(p_folder, PACKAGE_TGZ_NAME)
        client.run_command('gzip -d "{}"'.format(tgz))
        client.run_command('tar tvf "{}"'.format(os.path.join(p_folder, "conan_package.tar")))
        lines = str(client.out).splitlines()
        """
-rw-r--r-- 0/0               8 1970-01-01 01:00 file.txt
lrw-r--r-- 0/0               0 1970-01-01 01:00 link.txt -> file.txt
        """

        self.assertIn("link.txt", " ".join(lines))
        for line in lines:
            if ".txt" not in line:
                continue

            size = int([i for i in line.split(" ") if i][2])
            if "link.txt" in line:
                self.assertEqual(int(size), 0)
            elif "file.txt":
                self.assertGreater(int(size), 0)



# coding=utf-8

import os
import unittest

from mock import patch

from conans.client.hook_manager import HookManager
from conans.model.ref import ConanFileReference
from conans.paths import CONAN_MANIFEST
from conans.test.utils.tools import TurboTestClient


class PostPackageTestCase(unittest.TestCase):

    def test_create_command(self):
        """ Test that 'post_package' hook is called before computing the manifest
        """
        t = TurboTestClient()
        filename = "hook_file"

        def post_package_hook(conanfile, **kwargs):
            # There shouldn't be a manifest yet
            post_package_hook.manifest_path = os.path.join(conanfile.package_folder, CONAN_MANIFEST)
            self.assertFalse(os.path.exists(post_package_hook.manifest_path))
            # Add a file
            open(os.path.join(conanfile.package_folder, filename), "w").close()

        def mocked_load_hooks(hook_manager):
            hook_manager.hooks["post_package"] = [("_", post_package_hook)]

        with patch.object(HookManager, "load_hooks", new=mocked_load_hooks):
            pref = t.create(ConanFileReference.loads("name/version@user/channel"))

        # Check that we are considering the same file
        package_layout = t.cache.package_layout(pref.ref)
        self.assertEqual(post_package_hook.manifest_path,
                         os.path.join(package_layout.package(pref), CONAN_MANIFEST))
        # Now the file exists and contains info about created file
        self.assertTrue(os.path.exists(post_package_hook.manifest_path))
        with open(post_package_hook.manifest_path) as f:
            content = f.read()
            self.assertIn(filename, content)

    def test_export_pkg_command(self):
        """ Test that 'post_package' hook is called before computing the manifest
        """
        t = TurboTestClient()
        filename = "hook_file"

        def post_package_hook(conanfile, **kwargs):
            # There shouldn't be a manifest yet
            post_package_hook.manifest_path = os.path.join(conanfile.package_folder, CONAN_MANIFEST)
            self.assertFalse(os.path.exists(post_package_hook.manifest_path))
            # Add a file
            open(os.path.join(conanfile.package_folder, filename), "w").close()

        def mocked_load_hooks(hook_manager):
            hook_manager.hooks["post_package"] = [("_", post_package_hook)]

        with patch.object(HookManager, "load_hooks", new=mocked_load_hooks):
            pref = t.export_pkg(ref=ConanFileReference.loads("name/version@user/channel"),
                                args="--package-folder=.")

        # Check that we are considering the same file
        package_layout = t.cache.package_layout(pref.ref)
        self.assertEqual(post_package_hook.manifest_path,
                         os.path.join(package_layout.package(pref), CONAN_MANIFEST))
        # Now the file exists and contains info about created file
        self.assertTrue(os.path.exists(post_package_hook.manifest_path))
        with open(post_package_hook.manifest_path) as f:
            content = f.read()
            self.assertIn(filename, content)



import json
import textwrap
import unittest

import pytest

from conans.test.utils.tools import TestClient, GenConanfile
from conans.util.env_reader import get_env


class LockRecipeTest(unittest.TestCase):

    def test_error_pass_base(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . pkg/0.1@")
        client.save({"conanfile.py": GenConanfile().with_require("pkg/0.1")})
        client.run("lock create conanfile.py --base --lockfile-out=conan.lock")
        client.run("install . --lockfile=conan.lock", assert_error=True)
        self.assertIn("Lockfiles with --base do not contain profile information, "
                      "cannot be used. Create a full lockfile", client.out)

    def test_lock_recipe(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile().with_setting("os")})
        client.run("create . pkg/0.1@ -s os=Windows")
        self.assertIn("pkg/0.1:3475bd55b91ae904ac96fde0f106a136ab951a5e - Build", client.out)
        self.assertIn("pkg/0.1: Created package revision d0f0357277b3417d3984b5a9a85bbab6",
                      client.out)
        client.run("create . pkg/0.1@ -s os=Linux")
        self.assertIn("pkg/0.1:cb054d0b3e1ca595dc66bc2339d40f1f8f04ab31 - Build", client.out)
        self.assertIn("pkg/0.1: Created package revision 9e99cfd92d0d7df79d687b01512ce844",
                      client.out)

        client.save({"conanfile.py": GenConanfile().with_require("pkg/0.1")})
        client.run("lock create conanfile.py --base --lockfile-out=base.lock")
        lock = json.loads(client.load("base.lock"))
        self.assertEqual(2, len(lock["graph_lock"]["nodes"]))
        pkg_node = lock["graph_lock"]["nodes"]["1"]
        if client.cache.config.revisions_enabled:
            self.assertEqual(pkg_node["ref"], "pkg/0.1#f096d7d54098b7ad7012f9435d9c33f3")
        else:
            self.assertEqual(pkg_node["ref"], "pkg/0.1")
        client.run("lock create conanfile.py -s os=Linux "
                   "--lockfile-out=linux.lock --lockfile=base.lock")
        lock = json.loads(client.load("linux.lock"))
        pkg_node = lock["graph_lock"]["nodes"]["1"]
        if client.cache.config.revisions_enabled:
            self.assertEqual(pkg_node["ref"], "pkg/0.1#f096d7d54098b7ad7012f9435d9c33f3")
            self.assertEqual(pkg_node["package_id"], "cb054d0b3e1ca595dc66bc2339d40f1f8f04ab31")
            self.assertEqual(pkg_node["prev"], "9e99cfd92d0d7df79d687b01512ce844")
        else:
            self.assertEqual(pkg_node["ref"], "pkg/0.1")
            self.assertEqual(pkg_node["package_id"], "cb054d0b3e1ca595dc66bc2339d40f1f8f04ab31")
            self.assertEqual(pkg_node["prev"], "0")
        self.assertEqual(pkg_node["options"], "")
        self.assertIsNone(pkg_node.get("modified"))

        client.run("lock create conanfile.py -s os=Windows "
                   "--lockfile-out=windows.lock --lockfile=base.lock")
        lock = json.loads(client.load("windows.lock"))
        pkg_node = lock["graph_lock"]["nodes"]["1"]
        if client.cache.config.revisions_enabled:
            self.assertEqual(pkg_node["ref"], "pkg/0.1#f096d7d54098b7ad7012f9435d9c33f3")
            self.assertEqual(pkg_node["package_id"], "3475bd55b91ae904ac96fde0f106a136ab951a5e")
            self.assertEqual(pkg_node["prev"], "d0f0357277b3417d3984b5a9a85bbab6")
        else:
            self.assertEqual(pkg_node["ref"], "pkg/0.1")
            self.assertEqual(pkg_node["package_id"], "3475bd55b91ae904ac96fde0f106a136ab951a5e")
            self.assertEqual(pkg_node["prev"], "0")
        self.assertEqual(pkg_node["options"], "")

        # Now it is possible to obtain the base one again from the full ones
        client.run("lock create conanfile.py --base "
                   "--lockfile-out=windows_base.lock --lockfile=windows.lock")
        self.assertEqual(client.load("windows_base.lock"), client.load("base.lock"))
        # Now it is possible to obtain the base one again from the full ones
        client.run("lock create conanfile.py --base "
                   "--lockfile-out=linux_base.lock --lockfile=linux.lock")
        self.assertEqual(client.load("linux_base.lock"), client.load("base.lock"))

    def test_lock_recipe_from_partial(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . LibA/1.0@")
        client.save({"conanfile.py": GenConanfile().with_require("LibA/[>=1.0]")})
        client.run("create . LibB/1.0@")
        client.run("lock create --reference=LibB/1.0 --lockfile-out=base.lock --base")
        client.run("lock create --reference=LibB/1.0 --lockfile=base.lock --lockfile-out=libb.lock")

        client.save({"conanfile.py": GenConanfile()})
        client.run("create . LibA/1.0.1@")

        client.save({"conanfile.py": GenConanfile().with_require("LibB/1.0")})

        for lock in ("base.lock", "libb.lock"):
            client.run("lock create conanfile.py --name=LibC --version=1.0 --lockfile=%s "
                       "--lockfile-out=full.lock --base" % lock)
            self.assertIn("LibA/1.0 from local cache - Cache", client.out)

            lock = json.loads(client.load("full.lock"))
            for id_, ref, rrev in ("1", "LibB/1.0", "6e5c7369c3d3f7a7a5a60ddec16a941f"), \
                                  ("2", "LibA/1.0", "f3367e0e7d170aa12abccb175fee5f97"):
                pkg_node = lock["graph_lock"]["nodes"][id_]
                if client.cache.config.revisions_enabled:
                    self.assertEqual(pkg_node["ref"], "%s#%s" % (ref, rrev))
                else:
                    self.assertEqual(pkg_node["ref"], ref)
                self.assertIsNone(pkg_node.get("package_id"))
                self.assertIsNone(pkg_node.get("prev"))
                self.assertIsNone(pkg_node.get("options"))

    def test_conditional_lock_recipe(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile()})
        client.run("create . common/0.1@")
        client.run("create . win/0.1@")
        client.run("create . linux/0.1@")

        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                settings = "os"
                requires = "common/0.1"
                def requirements(self):
                    if self.settings.os == "Windows":
                        self.requires("win/0.1")
                    else:
                        self.requires("linux/0.1")
            """)
        client.save({"conanfile.py": conanfile})
        client.run("lock create conanfile.py --base -s os=Windows --lockfile-out=conan.lock")
        lock = json.loads(client.load("conan.lock"))
        self.assertEqual(3, len(lock["graph_lock"]["nodes"]))
        common = lock["graph_lock"]["nodes"]["1"]
        win = lock["graph_lock"]["nodes"]["2"]
        if client.cache.config.revisions_enabled:
            self.assertEqual(common["ref"], "common/0.1#f3367e0e7d170aa12abccb175fee5f97")
            self.assertEqual(win["ref"], "win/0.1#f3367e0e7d170aa12abccb175fee5f97")
        else:
            self.assertEqual(common["ref"], "common/0.1")
            self.assertEqual(win["ref"], "win/0.1")
        self.assertIsNone(common.get("package_id"))
        self.assertIsNone(common.get("prev"))
        self.assertIsNone(common.get("options"))
        self.assertIsNone(win.get("package_id"))
        self.assertIsNone(win.get("prev"))
        self.assertIsNone(win.get("options"))

        client.run("lock create conanfile.py -s os=Linux "
                   "--lockfile-out=linux.lock --lockfile=conan.lock")
        lock = json.loads(client.load("linux.lock"))
        self.assertEqual(3, len(lock["graph_lock"]["nodes"]))
        common = lock["graph_lock"]["nodes"]["1"]
        linux = lock["graph_lock"]["nodes"]["3"]
        self.assertNotIn("2", lock["graph_lock"]["nodes"])
        if client.cache.config.revisions_enabled:
            self.assertEqual(common["ref"], "common/0.1#f3367e0e7d170aa12abccb175fee5f97")
            self.assertEqual(common["prev"], "83c38d3b4e5f1b8450434436eec31b00")
            self.assertEqual(linux["ref"], "linux/0.1#f3367e0e7d170aa12abccb175fee5f97")
            self.assertEqual(linux["prev"], "83c38d3b4e5f1b8450434436eec31b00")
        else:
            self.assertEqual(common["ref"], "common/0.1")
            self.assertEqual(common["prev"], "0")
            self.assertEqual(linux["ref"], "linux/0.1")
            self.assertEqual(linux["prev"], "0")
        self.assertEqual(common["options"], "")
        self.assertEqual(common["package_id"], "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9")
        self.assertEqual(linux["package_id"], "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9")
        self.assertEqual(linux["options"], "")

        client.run("lock create conanfile.py -s os=Windows "
                   "--lockfile-out=windows.lock --lockfile=conan.lock")
        lock = json.loads(client.load("windows.lock"))
        self.assertEqual(3, len(lock["graph_lock"]["nodes"]))
        common = lock["graph_lock"]["nodes"]["1"]
        win = lock["graph_lock"]["nodes"]["2"]
        if client.cache.config.revisions_enabled:
            self.assertEqual(common["ref"], "common/0.1#f3367e0e7d170aa12abccb175fee5f97")
            self.assertEqual(common["prev"], "83c38d3b4e5f1b8450434436eec31b00")
            self.assertEqual(win["ref"], "win/0.1#f3367e0e7d170aa12abccb175fee5f97")
            self.assertEqual(win["prev"], "83c38d3b4e5f1b8450434436eec31b00")
        else:
            self.assertEqual(common["ref"], "common/0.1")
            self.assertEqual(common["prev"], "0")
            self.assertEqual(win["ref"], "win/0.1")
            self.assertEqual(win["prev"], "0")
        self.assertEqual(common["package_id"], "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9")
        self.assertEqual(common["options"], "")
        self.assertEqual(win["package_id"], "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9")
        self.assertEqual(win["options"], "")

    @pytest.mark.skipif(not get_env("TESTING_REVISIONS_ENABLED", False), reason="Only revisions")
    def test_lose_rrev(self):
        # https://github.com/conan-io/conan/issues/7595
        client = TestClient()
        client.run("config set general.default_package_id_mode=full_package_mode")
        files = {
            "pkga/conanfile.py": GenConanfile(),
            "pkgb/conanfile.py": GenConanfile().with_require("liba/[*]"),
        }
        client.save(files)

        client.run("create pkga liba/0.1@")

        client.run("lock create pkgb/conanfile.py --name=libb --version=0.1 "
                   "--lockfile-out=base.lock --base")

        client.run("export pkgb libb/0.1@ --lockfile=base.lock --lockfile-out=libb_base.lock")
        client.run("lock create --reference=libb/0.1@ --lockfile=libb_base.lock "
                   "--lockfile-out=libb_release.lock --build=missing")
        libb_release = client.load("libb_release.lock")
        self.assertIn('"ref": "libb/0.1#c2a641589d4b617387124f011905a97b"', libb_release)

        client.run("create pkgb libb/0.1@ --lockfile=libb_release.lock")
        self.assertIn("libb/0.1: Created package", client.out)

    @pytest.mark.skipif(not get_env("TESTING_REVISIONS_ENABLED", False), reason="Only revisions")
    def test_missing_configuration(self):
        client = TestClient()
        client.run("config set general.default_package_id_mode=package_revision_mode")
        client.save({"conanfile.py": GenConanfile().with_setting("os")})
        client.run("create . liba/0.1@ -s os=Windows")
        self.assertIn("liba/0.1:3475bd55b91ae904ac96fde0f106a136ab951a5e - Build", client.out)
        self.assertIn("liba/0.1: Created package revision d0f0357277b3417d3984b5a9a85bbab6",
                      client.out)

        client.save({"conanfile.py": GenConanfile().with_require("liba/0.1")})
        client.run("export . libb/0.1@")
        client.run("lock create --reference=libb/0.1 --base --lockfile-out=conan.lock -s os=Windows")

        client.run("lock create --reference=libb/0.1 -s os=Windows "
                   "--lockfile-out=windows.lock --lockfile=conan.lock "
                   "--build=libb/0.1 --build=missing")
        self.assertIn("libb/0.1:d9a360017881eddb68099b9a3573a4c0d39f3df5 - Build", client.out)

        client.run("lock create --reference=libb/0.1 -s os=Linux "
                   "--lockfile-out=linux.lock --lockfile=conan.lock "
                   "--build=libb/0.1 --build=missing")
        self.assertIn("libb/0.1:Package_ID_unknown - Unknown", client.out)

    @pytest.mark.skipif(not get_env("TESTING_REVISIONS_ENABLED", False), reason="Only revisions")
    def test_missing_configuration_build_require(self):
        client = TestClient()
        client.run("config set general.default_package_id_mode=package_revision_mode")

        client.save({"conanfile.py": GenConanfile()})
        client.run("create . cmake/1.0@")
        client.save({"conanfile.py": GenConanfile().with_setting("os"),
                     "myprofile": "[build_requires]\ncmake/1.0"})
        client.run("create . liba/0.1@ -s os=Windows --profile=myprofile")
        self.assertIn("liba/0.1:3475bd55b91ae904ac96fde0f106a136ab951a5e - Build", client.out)
        self.assertIn("liba/0.1: Created package revision d0f0357277b3417d3984b5a9a85bbab6",
                      client.out)

        client.save({"conanfile.py": GenConanfile().with_require("liba/0.1")})
        client.run("lock create conanfile.py --name=libb --version=0.1 --base "
                   "--lockfile-out=conan.lock --profile=myprofile -s os=Windows --build")

        client.run("export . libb/0.1@ --lockfile=conan.lock --lockfile-out=conan.lock")

        client.run("lock create --reference=libb/0.1 -s os=Windows "
                   "--lockfile-out=windows.lock --lockfile=conan.lock "
                   "--build=libb/0.1 --build=missing --profile=myprofile")
        self.assertIn("liba/0.1:3475bd55b91ae904ac96fde0f106a136ab951a5e - Cache", client.out)
        self.assertIn("libb/0.1:d9a360017881eddb68099b9a3573a4c0d39f3df5 - Build", client.out)
        self.assertIn("cmake/1.0:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Cache", client.out)

        client.run("lock create --reference=libb/0.1 -s os=Linux "
                   "--lockfile-out=linux.lock --lockfile=conan.lock "
                   "--build=libb/0.1 --build=missing --profile=myprofile")
        self.assertNotIn("ERROR: No package matching 'libb/0.1' pattern", client.out)
        self.assertIn("liba/0.1:cb054d0b3e1ca595dc66bc2339d40f1f8f04ab31 - Build", client.out)
        self.assertIn("libb/0.1:Package_ID_unknown - Unknown", client.out)
        self.assertIn("cmake/1.0:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Cache", client.out)



import platform
import textwrap
import unittest

from conans.test.utils.tools import TestClient, GenConanfile


class CollectLibsTest(unittest.TestCase):

    def test_collect_libs(self):
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                exports_sources = "*"
                def package(self):
                    self.copy("*", dst="lib")
                def package_info(self):
                    from conans import tools
                    self.cpp_info.libs = tools.collect_libs(self)
            """)
        client = TestClient()
        lib_name = "mylibname.%s" % ("a" if platform.system() != "Windows" else "lib")
        client.save({"conanfile.py": conanfile,
                     lib_name: ""})
        client.run("create . mylib/0.1@user/channel")

        # reusing the binary already in cache
        client.save({"conanfile.py": GenConanfile().with_require("mylib/0.1@user/channel")},
                    clean_first=True)
        client.run('install . -g cmake')
        conanbuildinfo = client.load("conanbuildinfo.cmake")
        self.assertIn("set(CONAN_LIBS_MYLIB mylibname)", conanbuildinfo)
        self.assertIn("set(CONAN_LIBS mylibname ${CONAN_LIBS})", conanbuildinfo)

        # rebuilding the binary in cache
        client.run('remove "*" -p -f')
        client.run('install . --build -g cmake')
        conanbuildinfo = client.load("conanbuildinfo.cmake")
        self.assertIn("set(CONAN_LIBS_MYLIB mylibname)", conanbuildinfo)
        self.assertIn("set(CONAN_LIBS mylibname ${CONAN_LIBS})", conanbuildinfo)



import textwrap

lib_h = textwrap.dedent("""
    int some_function(int value);
""")

lib_cpp = textwrap.dedent("""
    #include "lib.h"
    #include <iostream>

    int some_function(int value) {
        std::cout << "some_function(value=" << value << ")" << std::endl;
        return 42;
    }
""")

cmakelists = textwrap.dedent("""
    cmake_minimum_required(VERSION 2.8.12)  # TODO: Define minimun required here
    project(AndroidLibrary CXX)

    add_library(library lib.h lib.cpp)
    set_target_properties(library PROPERTIES PUBLIC_HEADER lib.h)

    install(TARGETS library
        RUNTIME DESTINATION bin
        LIBRARY DESTINATION lib
        ARCHIVE DESTINATION lib
        PUBLIC_HEADER DESTINATION include
    )
""")


def create_library(client):
    client.save({
        'lib.h': lib_h,
        'lib.cpp': lib_cpp,
        'CMakeLists.txt': cmakelists
    })



# coding=utf-8

import os
import unittest

from conans.model.ref import ConanFileReference
from conans.test.utils.tools import TestClient, GenConanfile
from conans.util.files import mkdir


class TransitiveEditableTest(unittest.TestCase):

    def test_transitive_editables(self):
        # https://github.com/conan-io/conan/issues/4445
        libc_ref = ConanFileReference.loads("LibC/0.1@user/testing")
        libb_ref = ConanFileReference.loads("LibB/0.1@user/testing")

        client = TestClient()
        conanfileC = GenConanfile()
        client.save({"conanfile.py": str(conanfileC)})
        client.run("editable add . LibC/0.1@user/testing")

        client2 = TestClient(client.cache_folder)
        conanfileB = GenConanfile().with_name("LibB").with_version("0.1").with_require(libc_ref)

        client2.save({"conanfile.py": str(conanfileB)})
        client2.run("create . user/testing")

        conanfileA = GenConanfile().with_name("LibA").with_version("0.1")\
                                   .with_require(libb_ref)\
                                   .with_require(libc_ref)
        client2.save({"conanfile.py": str(conanfileA)})
        client2.run("install .")
        client2.current_folder = os.path.join(client2.current_folder, "build")
        mkdir(client2.current_folder)
        client2.run("install ..")



# -*- coding: utf-8 -*-

import json
import os
import textwrap
import unittest

from conans.test.utils.tools import TestClient
from conans.util.files import save


class GeneratorFromCacheTest(unittest.TestCase):
    """
    Tests user defined generators loaded from the generators folder in
    the client cache.
    """

    def test_install_cmdline(self):
        """
        Test the availability of user-defined generators from the command-line
        using `install -g`
        """
        client = TestClient(cache_autopopulate=True)

        # Set up generator. This generator will interact with another package.
        test_generator_contents = textwrap.dedent("""
            import json

            from conans.model import Generator

            class user_defined(Generator):
                @property
                def filename(self):
                    return "userdefined.json"

                @property
                def content(self):
                    return json.dumps(self.deps_env_info.vars)
            """)

        # Save generator to cache_folder/generators.
        generator_path = os.path.join(client.cache.generators_path, "user_defined_generator.py")
        save(generator_path, test_generator_contents)

        conanfile_py = textwrap.dedent("""
            from conans import ConanFile

            class HelloConan(ConanFile):
                build_policy = "always"

                def package_info(self):
                    self.env_info.MY_ENV_VAR1 = "foo"
                    self.env_info.MY_ENV_VAR2 = "bar"
                    self.env_info.MY_ENV_VAR3 = "baz"
                    """)
        client.save({"conanfile.py": conanfile_py})

        # Test the install using a reference
        client.run("export . Hello/0.1@lasote/testing")
        client.run("install Hello/0.1@lasote/testing -g user_defined")

        data = client.load("userdefined.json")
        json_data = json.loads(data)
        self.assertEqual(json_data["MY_ENV_VAR1"], "foo")
        self.assertEqual(json_data["MY_ENV_VAR2"], "bar")
        self.assertEqual(json_data["MY_ENV_VAR3"], "baz")

    def test_install_conanfile(self):
        """
        Test the availability of user-defined generators from a conanfile
        """
        client = TestClient()

        # Set up generator
        test_generator_contents = textwrap.dedent("""
            from conans import ConanFile
            from conans.model import Generator

            class user_defined(Generator):
                @property
                def filename(self):
                    return "userdefined.txt"

                @property
                def content(self):
                    return "user_defined contents"
            """)
        # Save generator to cache_folder/generators.
        generator_path = os.path.join(client.cache.generators_path, "user_defined_generator.py")
        save(generator_path, test_generator_contents)

        conanfile_py = textwrap.dedent("""
            from conans import ConanFile, tools

            class HelloConan(ConanFile):
                generators = "user_defined"

                def package(self):
                    self.output.info("create_conanfile_test: {}"
                                      .format(tools.load("userdefined.txt")))
                """)
        client.save({"conanfile.py": conanfile_py})

        # Test the install using a conanfile
        client.run("install . --build")

        data = client.load("userdefined.txt")
        self.assertEqual(data, "user_defined contents")

        # Test the install using a conanfile
        client.run("create . Hello/0.1@lasote/testing")
        self.assertIn("Generator user_defined created userdefined.txt", client.out)
        self.assertIn("create_conanfile_test: user_defined contents", client.out)

        client.run("install Hello/0.1@lasote/testing --build")



import textwrap
import pytest

from conans.test.utils.tools import TestClient

consumer = textwrap.dedent("""
from conans import ConanFile
from conan.tools.cmake import CMake

class Consumer(ConanFile):
    name = "consumer"
    version = "1.0"
    settings = "os", "compiler", "build_type", "arch"
    generators = "CMakeDeps", "CMakeToolchain"
    exports_sources = ["CMakeLists.txt"]
    requires = "hello/1.0"

    def build(self):
        cmake = CMake(self)
        cmake.configure()
        cmake.build()
""")


@pytest.mark.tool_cmake
def test_global_alias():
    conanfile = textwrap.dedent("""
    from conans import ConanFile

    class Hello(ConanFile):
        name = "hello"
        version = "1.0"
        settings = "os", "compiler", "build_type", "arch"

        def package_info(self):
            # the default global target is "hello::hello"
            self.cpp_info.set_property("cmake_target_aliases", ["hello"])
    """)

    cmakelists = textwrap.dedent("""
    cmake_minimum_required(VERSION 3.15)
    project(test)

    find_package(hello REQUIRED)
    get_target_property(link_libraries hello INTERFACE_LINK_LIBRARIES)
    message("hello link libraries: ${link_libraries}")
    """)

    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create .")

    client.save({"conanfile.py": consumer, "CMakeLists.txt": cmakelists})
    client.run("create .")

    assert "hello link libraries: hello::hello" in client.out


@pytest.mark.tool_cmake
def test_component_alias():
    conanfile = textwrap.dedent("""
    from conans import ConanFile

    class Hello(ConanFile):
        name = "hello"
        version = "1.0"
        settings = "os", "compiler", "build_type", "arch"

        def package_info(self):
            self.cpp_info.components["buy"].set_property("cmake_target_aliases",
                ["hola::adios"])
    """)

    cmakelists = textwrap.dedent("""
    cmake_minimum_required(VERSION VERSION 3.15)
    project(test)

    find_package(hello REQUIRED)
    get_target_property(link_libraries hola::adios INTERFACE_LINK_LIBRARIES)
    message("hola::adios link libraries: ${link_libraries}")
    """)

    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create .")

    client.save({"conanfile.py": consumer, "CMakeLists.txt": cmakelists})
    client.run("create .")

    assert "hola::adios link libraries: hello::buy" in client.out


@pytest.mark.tool_cmake
def test_custom_name():
    conanfile = textwrap.dedent("""
    from conans import ConanFile

    class Hello(ConanFile):
        name = "hello"
        version = "1.0"
        settings = "os", "compiler", "build_type", "arch"

        def package_info(self):
            self.cpp_info.set_property("cmake_target_name", "ola::comprar")
            self.cpp_info.set_property("cmake_target_aliases", ["hello"])
    """)

    cmakelists = textwrap.dedent("""
    cmake_minimum_required(VERSION 3.15)
    project(test)

    find_package(hello REQUIRED)
    get_target_property(link_libraries hello INTERFACE_LINK_LIBRARIES)
    message("hello link libraries: ${link_libraries}")
    """)

    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create .")

    client.save({"conanfile.py": consumer, "CMakeLists.txt": cmakelists})
    client.run("create .")

    assert "hello link libraries: ola::comprar" in client.out


@pytest.mark.tool_cmake
def test_collide_global_alias():
    conanfile = textwrap.dedent("""
    from conans import ConanFile

    class Hello(ConanFile):
        name = "hello"
        version = "1.0"
        settings = "os", "compiler", "build_type", "arch"

        def package_info(self):
            # the default global target is "hello::hello"
            self.cpp_info.set_property("cmake_target_aliases", ["hello::hello"])
    """)

    cmakelists = textwrap.dedent("""
    cmake_minimum_required(VERSION 3.15)
    project(test)

    find_package(hello REQUIRED)
    """)

    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create .")

    client.save({"conanfile.py": consumer, "CMakeLists.txt": cmakelists})
    client.run("create .")

    assert "Target name 'hello::hello' already exists." in client.out


@pytest.mark.tool_cmake
def test_collide_component_alias():
    conanfile = textwrap.dedent("""
    from conans import ConanFile

    class Hello(ConanFile):
        name = "hello"
        version = "1.0"
        settings = "os", "compiler", "build_type", "arch"

        def package_info(self):
            self.cpp_info.components["buy"].set_property("cmake_target_aliases", ["hello::buy"])
    """)

    cmakelists = textwrap.dedent("""
    cmake_minimum_required(VERSION 3.15)
    project(test)

    find_package(hello REQUIRED)
    """)

    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("create .")

    client.save({"conanfile.py": consumer, "CMakeLists.txt": cmakelists})
    client.run("create .")

    assert "Target name 'hello::buy' already exists." in client.out



import os
import unittest

from conans.client import tools
from conans.test.utils.test_files import temp_folder
from conans.test.utils.tools import TestClient
from conans.util.files import save


class ClientCertsTest(unittest.TestCase):
    class MyHttpRequester(object):
        def __init__(self, *args, **kwargs):
            pass

        def get(self, _, **kwargs):
            return kwargs.get("cert", None)

    def test_pic_client_certs(self):
        client = TestClient(requester_class=self.MyHttpRequester)
        self.assertIsNone(client.requester.get("url"))

        config = client.cache.config
        tools.save(config.client_cert_path, "Fake cert")

        self.assertEqual(client.requester.get("url"), client.cache.config.client_cert_path)

        tools.save(config.client_cert_path, "Fake cert")
        tools.save(config.client_cert_key_path, "Fake key")
        self.assertEqual(client.requester.get("url"), (config.client_cert_path,
                                                       config.client_cert_key_path))

        # assert that the cacert file is created
        self.assertTrue(os.path.exists(config.cacert_path))

    def test_pic_custom_path_client_certs(self):
        folder = temp_folder()
        mycert_path = os.path.join(folder, "mycert.crt")
        mykey_path = os.path.join(folder, "mycert.key")
        save(mycert_path, "Fake Cert")
        save(mykey_path, "Fake Key")

        client = TestClient(requester_class=self.MyHttpRequester)
        client.run('config set general.client_cert_path="%s"' % mycert_path)
        client.run('config set general.client_cert_key_path="%s"' % mykey_path)

        self.assertEqual(client.requester.get("url"), (mycert_path, mykey_path))

        # assert that the cacert file is created
        self.assertTrue(os.path.exists(client.cache.config.cacert_path))



# coding=utf-8

import unittest
import os

from conans.client.conf.config_installer import tmp_config_install_folder
from conans.test.utils.tools import TestClient


class InstallFolderTests(unittest.TestCase):

    def test_unique_install_folder(self):
        """ Validate if tmp_config_install_folder is removing old folder before creating a new one

        tmp_config_install_folder must create the same folder, but all items must be exclude when a
        new folder is created.
        """
        client = TestClient()

        with tmp_config_install_folder(client.cache) as tmp_folder_first:
            temp_file = os.path.join(tmp_folder_first, "foobar.txt")
            open(temp_file, "w+")
            with tmp_config_install_folder(client.cache) as tmp_folder_second:
                self.assertEqual(tmp_folder_first, tmp_folder_second)
                self.assertFalse(os.path.exists(temp_file))



import os
import platform
import sys
import textwrap

import pytest

from conans.client.tools.apple import XCRun, to_apple_arch
from conans.test.utils.tools import TestClient

_conanfile_py = textwrap.dedent("""
from conan import ConanFile
from conan.tools.meson import Meson, MesonToolchain


class App(ConanFile):
    settings = "os", "arch", "compiler", "build_type"
    options = {"shared": [True, False], "fPIC": [True, False]}
    default_options = {"shared": False, "fPIC": True}

    def layout(self):
        self.folders.build = "build"

    def config_options(self):
        if self.settings.os == "Windows":
            del self.options.fPIC

    def generate(self):
        tc = MesonToolchain(self)
        tc.generate()

    def build(self):
        meson = Meson(self)
        meson.configure()
        meson.build()
""")

_meson_build_objc = textwrap.dedent("""
project('tutorial', 'objc')
executable('demo', 'main.m', link_args: ['-framework', 'Foundation'])
""")


@pytest.mark.tool_meson
@pytest.mark.skipif(sys.version_info.major == 2, reason="Meson not supported in Py2")
@pytest.mark.skipif(platform.system() != "Darwin", reason="requires Xcode")
def test_apple_meson_toolchain_native_compilation_objective_c():
    profile = textwrap.dedent("""
    [settings]
    os = Macos
    arch = x86_64
    compiler = apple-clang
    compiler.version = 12.0
    compiler.libcxx = libc++
    """)
    app = textwrap.dedent("""
    #import <Foundation/Foundation.h>

    int main(int argc, const char * argv[]) {
        @autoreleasepool {
            // insert code here...
            NSLog(@"Hello, World!");
        }
        return 0;
    }
    """)
    t = TestClient()
    t.save({"conanfile.py": _conanfile_py,
            "meson.build": _meson_build_objc,
            "main.m": app,
            "macos_pr": profile})

    t.run("install . -pr macos_pr")
    t.run("build .")
    t.run_command("./demo", cwd=os.path.join(t.current_folder, "build"))
    assert "Hello, World!" in t.out


@pytest.mark.parametrize("arch, os_, os_version, sdk", [
    ('armv8', 'iOS', '10.0', 'iphoneos'),
    ('armv7', 'iOS', '10.0', 'iphoneos'),
    ('x86', 'iOS', '10.0', 'iphonesimulator'),
    ('x86_64', 'iOS', '10.0', 'iphonesimulator'),
    ('armv8', 'Macos', None, None)  # MacOS M1
])
@pytest.mark.tool_meson
@pytest.mark.skipif(sys.version_info.major == 2, reason="Meson not supported in Py2")
@pytest.mark.skipif(platform.system() != "Darwin", reason="requires Xcode")
def test_apple_meson_toolchain_cross_compiling_and_objective_c(arch, os_, os_version, sdk):
    profile = textwrap.dedent("""
    include(default)

    [settings]
    os = {os}
    os.version = {os_version}
    os.sdk = {os_sdk}
    arch = {arch}
    compiler = apple-clang
    compiler.version = 12.0
    compiler.libcxx = libc++

    [conf]
    tools.apple:sdk_path={sdk_path}
    """)

    xcrun = XCRun(None, sdk)
    sdk_path = xcrun.sdk_path
    app = textwrap.dedent("""
    #import <Foundation/Foundation.h>

    int main(int argc, const char * argv[]) {
        @autoreleasepool {
            // insert code here...
            NSLog(@"Hello, World!");
        }
        return 0;
    }
    """)
    profile = profile.format(
        os=os_,
        os_version=os_version,
        os_sdk=sdk,
        arch=arch,
        sdk_path=sdk_path)

    t = TestClient()
    t.save({"conanfile.py": _conanfile_py,
            "meson.build": _meson_build_objc,
            "main.m": app,
            "profile_host": profile})

    t.run("install . --profile:build=default --profile:host=profile_host")
    t.run("build .")
    assert "Objective-C compiler for the host machine: clang" in t.out

    demo = os.path.join(t.current_folder, "build", "demo")
    assert os.path.isfile(demo) is True

    lipo = xcrun.find('lipo')
    t.run_command('"%s" -info "%s"' % (lipo, demo))
    assert "architecture: %s" % to_apple_arch(arch) in t.out



import platform
import textwrap
import unittest

import pytest

from conan.tools.cmake import CMakeToolchain
from conans.test.utils.tools import TestClient
from ._utils import create_library


@pytest.mark.skipif(platform.system() != "Darwin", reason="Requires XCode")
class ToolchainiOSTestCase(unittest.TestCase):

    def setUp(self):
        self.t = TestClient()
        create_library(self.t)
        self._conanfile = textwrap.dedent("""
            from conans import ConanFile
            from conan.tools.cmake import CMake, CMakeToolchain

            class Library(ConanFile):
                name = 'hello'
                version = '1.0'
                settings = 'os', 'arch', 'compiler', 'build_type'
                exports_sources = 'hello.h', 'hello.cpp', 'CMakeLists.txt'
                options = {{'shared': [True, False]}}
                default_options = {{'shared': False}}

                def generate(self):
                    tc = CMakeToolchain(self, generator={generator})
                    tc.generate()

                def build(self):
                    cmake = CMake(self)
                    cmake.configure()
                    cmake.build()
                    self.run("lipo -info Release-iphoneos/libhello.a")

                def package(self):
                    cmake = CMake(self)
                    cmake.install()
            """)

        self.t.save({
            'ios_profile': textwrap.dedent("""
                [settings]
                os=iOS
                os.sdk=iphoneos
                os.version=12.0
                arch=armv8
                compiler=apple-clang
                compiler.version=12.0
                compiler.libcxx=libc++
                build_type=Release
            """)
        })

    @pytest.mark.tool_cmake(version="3.19")
    def test_xcode_generator(self):
        """ Simplest approach:
            https://cmake.org/cmake/help/latest/manual/cmake-toolchains.7.html#cross-compiling-for-ios-tvos-or-watchos
        """
        self.t.save({'conanfile.py': self._conanfile.format(generator='"Xcode"')})

        # Build in the cache
        self.t.run('create . --profile:build=default --profile:host=ios_profile')
        self.assertIn("Non-fat file: Release-iphoneos/libhello.a is architecture: arm64", self.t.out)

        # Build locally
        self.t.run('install . --profile:host=ios_profile --profile:build=default')
        self.t.run_command('cmake . -G"Xcode" -DCMAKE_TOOLCHAIN_FILE={}'.format(CMakeToolchain.filename))
        self.t.run_command('cmake --build . --config Release')
        self.t.run_command("lipo -info Release-iphoneos/libhello.a")
        self.assertIn("Non-fat file: Release-iphoneos/libhello.a is architecture: arm64", self.t.out)

    def test_unix_makefiles_generator(self):
        pass



from parameterized import parameterized

from conans.client import tools
from conans.paths import CONANFILE
from conans.test.utils.tools import TestClient


conanfile = """
from conans import ConanFile

class DRLException(Exception):
    pass

class ExceptionsTest(ConanFile):
    name = "exceptions"
    version = "0.1"

    def {method}(self):
        {method_contents}

    def _aux_method(self):
        raise DRLException('Oh! an error!')
"""


@parameterized.expand([(True,), (False,)])
def test_all_methods(direct):
    client = TestClient()
    if direct:
        throw = "raise DRLException('Oh! an error!')"
    else:
        throw = "self._aux_method()"
    for method in ["source", "build", "package", "package_info", "configure", "build_id",
                   "package_id", "requirements", "config_options", "layout", "generate", "export",
                   "export_sources"]:
        client.save({CONANFILE: conanfile.format(method=method, method_contents=throw)})
        client.run("create . ", assert_error=True)
        assert "exceptions/0.1: Error in %s() method, line 12" % method in client.out
        assert "DRLException: Oh! an error!" in client.out
        if not direct:
            assert "while calling '_aux_method', line 15" in client.out


def test_complete_traceback():
    client = TestClient()
    throw = "self._aux_method()"
    client.save({CONANFILE: conanfile.format(method="source", method_contents=throw)})
    with tools.environment_append({"CONAN_VERBOSE_TRACEBACK": "1"}):
        client.run("create . ", assert_error=True)
        assert "DRLException: Oh! an error!" in client.out
        assert "ERROR: Traceback (most recent call last):" in client.out



# coding=utf-8

import os
import unittest


from conans.test.utils.test_files import temp_folder
from conans.util.files import set_dirty, clean_dirty, set_dirty_context_manager, _DIRTY_FOLDER


class DirtyTest(unittest.TestCase):

    def setUp(self):
        """ Create temporary folder to save dirty state

        """
        self.temp_folder = temp_folder()
        self.dirty_folder = self.temp_folder + _DIRTY_FOLDER

    def test_set_dirty(self):
        """ Dirty flag must be created by set_dirty

        """
        set_dirty(self.temp_folder)
        self.assertTrue(os.path.exists(self.dirty_folder))

    def test_clean_dirty(self):
        """ Dirty flag must be cleaned by clean_dirty

        """
        set_dirty(self.temp_folder)
        self.assertTrue(os.path.exists(self.dirty_folder))
        clean_dirty(self.temp_folder)
        self.assertFalse(os.path.exists(self.dirty_folder))

    def test_set_dirty_context(self):
        """ Dirty context must remove lock before exiting

        """
        with set_dirty_context_manager(self.temp_folder):
            self.assertTrue(os.path.exists(self.dirty_folder))
        self.assertFalse(os.path.exists(self.dirty_folder))

    def test_interrupted_dirty_context(self):
        """ Broken context must preserve dirty state

            Raise an exception in middle of context. By default,
            dirty file is not removed.
        """
        try:
            with set_dirty_context_manager(self.temp_folder):
                self.assertTrue(os.path.exists(self.dirty_folder))
                raise RuntimeError()
        except RuntimeError:
            pass
        self.assertTrue(os.path.exists(self.dirty_folder))
